// 'use babel';

const _isset = function(attr,type) {
  try {
    var temp = attr instanceof type;
  } catch (e) {
    var temp = false;
  }
  return (attr !== null && attr !== undefined && (typeof attr === type || temp));
}

const Ajax = require('./externals/jquery-3.2.1.min').ajax;

exports.CouchDB = class {

  constructor(opts) {
    if(_isset(opts,'object') && _isset(opts.host,'string') && _isset(opts.port,'number') && _isset(opts.username,'string') && _isset(opts.password,'string')) {
      this.host = opts.host;
      this.port = opts.port;
      this.auth = {
        name: opts.username,
        password: opts.password
      };
    } else {
      throw new TypeError('Invalid arguments.');
    }
  }

  _build(type) {
    var base = 'http://'+this.host+':'+this.port+'/';
    switch (type) {
      // case 'login':
      //   return base+'_session';
      //   break;
      case 'dbs':
        return base+'_all_dbs';
        break;
      default:
        return base;
    }
  }

  allDbs(callback) {
    var _this = this;

    if(_isset(callback,'function')) {
      Ajax({
        async: true,
        url: _this._build('dbs'),
        method: 'GET',
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        },
        complete: function(dbs,status) {
          if(dbs.status == 200) {
            callback({
              status: 'done',
              msg: '',
              response: (typeof dbs.responseJSON=='object'?dbs.responseJSON:dbs.responseText)
            });
          } else {
            callback({
              status: 'error',
              msg: 'Invalid response status: '+dbs.status
            });
          }
        }
      });
    } else {
      var dbs = Ajax({
        async: false,
        url: _this._build('dbs'),
        method: 'GET',
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        }
      });
      if(dbs.status == 200) {
        return (typeof dbs.responseJSON=='object'?dbs.responseJSON:dbs.responseText);
      } else {
        return undefined;
      }
    }
  }

  getDb(identifier,callback) {

    if(!identifier) {
      throw new Error('Invalid Arguments given!');
    }

    var _this = this;

    if(_isset(callback,'function')) {
      Ajax({
        async: true,
        url: _this._build()+identifier,
        method: 'GET',
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        },
        complete: function(db,status) {
          if(db.status == 200) {
            callback({
              status: 'done',
              msg: '',
              response: (typeof db.responseJSON == 'object'?db.responseJSON:db.responseText)
            });
          } else {
            callback({
              status: 'error',
              msg: 'Invalid response status: '+db.status
            });
          }
        }
      });
    } else {
      var db = Ajax({
        async: false,
        url: _this._build()+identifier,
        method: 'GET',
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        }
      });
      if(db.status == 200) {
        return (typeof db.responseJSON == 'object'?db.responseJSON:db.responseText);
      } else {
        return undefined;
      }
    }
  }

  getDocs(identifier,callback) {

    if(!identifier) {
      throw new Error('Invalid Arguments given!');
    }

    var _this = this;

    if(_isset(callback,'function')) {
      Ajax({
        async: true,
        url: _this._build()+identifier+'/_all_docs',
        method: 'GET',
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        },
        complete: function(docs,status) {
          if(docs.status == 200) {
            callback({
              status: 'done',
              msg: '',
              response: (typeof docs.responseJSON == 'object'?docs.responseJSON:docs.responseText)
            });
          } else {
            callback({
              status: 'error',
              msg: 'Invalid response status: '+docs.status
            });
          }
        }
      });
    } else {
      var docs = Ajax({
        async: false,
        url: _this._build()+identifier+'/_all_docs',
        method: 'GET',
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        }
      });
      if(docs.status == 200) {
        return (typeof docs.responseJSON == 'object'?docs.responseJSON:docs.responseText);
      } else {
        return undefined;
      }
    }
  }

  postDoc(dbIdentifier,data,callback) {

    if(!dbIdentifier || !data) {
      throw new Error('Invalid Arguments given!');
    }

    var _this = this;

    if(_isset(callback,'function')) {
      Ajax({
        async: true,
        url: _this._build()+dbIdentifier,
        method: 'POST',
        contentType: 'application/json',
        data: JSON.stringify(data),
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        },
        complete: function(res,status) {
          switch (res.status) {
            case 201:
              // Document created and stored on disk
              callback({
                status: 'done',
                msg: 'Document created and stored on disk.',
                response: (typeof res.responseJSON == 'object'?res.responseJSON:res.responseText)
              });
              break;
            case 202:
              // Document data accepted, but not yet stored on disk
              callback({
                status: 'pending',
                msg: 'Document data received, but not yet stored.',
                response: (typeof res.responseJSON == 'object'?res.responseJSON:res.responseText)
              });
              break;
            case 400:
              // Invalid database name
              callback({
                status: 'error',
                msg: 'Invalid database name!'
              });
              // TODO: Autocreation of missing database as config option?
              break;
            case 401:
              // Write privileges required
              callback({
                status: 'error',
                msg: 'Write privileges required!'
              });
              break;
            case 404:
              // Database doesn’t exist
              callback({
                status: 'error',
                msg: 'Database doesn\'t exist!'
              });
              // TODO: Autocreation of missing database as config option?
              break;
            case 409:
              // A Conflicting Document with same ID already exists
              callback({
                status: 'error',
                msg: 'A conflicting document with same ID already exists!'
              });
              break;
            default:
              callback({
                status: 'error',
                msg: 'Invalid response status: '+res.status
              });
          }
        }
      });
    } else {
      var res = Ajax({
        async: false,
        url: _this._build()+dbIdentifier,
        method: 'POST',
        contentType: 'application/json',
        data: JSON.stringify(data),
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        }
      });

      switch (res.status) {
        case 201:
          // Document created and stored on disk
          return {
            status: 'done',
            msg: 'Document created and stored on disk.',
            response: (typeof res.responseJSON == 'object'?res.responseJSON:res.responseText)
          }
          break;
        case 202:
          // Document data accepted, but not yet stored on disk
          return {
            status: 'pending',
            msg: 'Document data received, but not yet stored.',
            response: (typeof res.responseJSON == 'object'?res.responseJSON:res.responseText)
          }
          break;
        case 400:
          // Invalid database name
          return {
            status: 'error',
            msg: 'Invalid database name!'
          }
          break;
        case 401:
          // Write privileges required
          return {
            status: 'error',
            msg: 'Write privileges required!'
          }
          break;
        case 404:
          // Database doesn’t exist
          return {
            status: 'error',
            msg: 'Database doesn\'t exist!'
          }
          break;
        case 409:
          // A Conflicting Document with same ID already exists
          return {
            status: 'error',
            msg: 'A conflicting document with same ID already exists!'
          }
          break;
        default:
          return {
            status: 'error',
            msg: 'Invalid response status: '+res.status
          }
      }
    }
  }

  getDoc(dbIdentifier,docIdentifier,callback) {

    if(!dbIdentifier || !docIdentifier) {
      throw new Error('Invalid Arguments given!');
    }

    var _this = this;

    if(_isset(callback,'function')) {
      Ajax({
        async: true,
        url: _this._build()+dbIdentifier+'/'+docIdentifier,
        method: 'GET',
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        },
        complete: function(doc,status) {
          switch (doc.status) {
            case 200:
              callback({
                status: 'done',
                msg: '',
                response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
              });
              break;
            case 304:
              // Document wasn’t modified since specified revision
              callback({
                status: 'done',
                msg: 'Document wasn’t modified since specified revision',
                response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
              });
              break;
            case 401:
              // Read privileges required
              callback({
                status: 'error',
                msg: 'Read privileges required!'
              });
              break;
            case 404:
              // Document not found
              callback({
                status: 'error',
                msg: 'Document not found!'
              });
              break;
            default:
              callback({
                status: 'error',
                msg: 'Invalid response status: '+doc.status
              });
          }
        }
      });
    } else {
      var doc = Ajax({
        async: false,
        url: _this._build()+dbIdentifier+'/'+docIdentifier,
        method: 'GET',
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        }
      });

      switch (doc.status) {
        case 200:
          return {
            status: 'done',
            msg: '',
            response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
          }
          break;
        case 304:
          // Document wasn’t modified since specified revision
          return {
            status: 'done',
            msg: 'Document wasn’t modified since specified revision',
            response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
          }
          break;
        case 401:
          // Read privileges required
          return {
            status: 'error',
            msg: 'Read privileges required!'
          }
          break;
        case 404:
          // Document not found
          return {
            status: 'error',
            msg: 'Document not found!'
          }
          break;
        default:
          return {
            status: 'error',
            msg: 'Invalid response status: '+doc.status
          }
      }
    }
  }

  deleteDoc(dbIdentifier,docIdentifier,revision,callback) {
    var _this = this;

    if(!revision || !dbIdentifier || !docIdentifier) {
      throw new Error('Invalid Arguments given!');
    }

    if(_isset(callback,'function')) {
      Ajax({
        async: true,
        url: _this._build()+dbIdentifier+'/'+docIdentifier,
        method: 'DELETE',
        contentType: 'application/json',
        data: JSON.stringify({
          rev: revision
        }),
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        },
        complete: function(doc,status) {
          switch (doc.status) {
            case 200:
              // Document successfully removed
              callback({
                status: 'done',
                msg: 'Document successfully removed.',
                response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
              });
              break;
            case 202:
              // Request was accepted, but changes are not yet stored on disk
              callback({
                status: 'pending',
                msg: 'Request received, but not applied.',
                response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
              });
              break;
            case 401:
              // Write privileges required
              callback({
                status: 'error',
                msg: 'Write privileges required!'
              });
              break;
            case 404:
              // Specified database or document ID doesn’t exists
              callback({
                status: 'error',
                msg: 'Specified database or document ID doesn’t exists!'
              });
              break;
            case 409:
              // Specified revision is not the latest for target document
              callback({
                status: 'error',
                msg: 'Specified revision is not the latest for target document!'
              });
              break;
            default:
              callback({
                status: 'error',
                msg: 'Invalid response status: '+doc.status
              });
          }
        }
      });
    } else {
      var doc = Ajax({
        async: false,
        url: _this._build()+dbIdentifier+'/'+docIdentifier,
        method: 'DELETE',
        contentType: 'application/json',
        data: JSON.stringify({
          rev: revision
        }),
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        }
      });

      switch (doc.status) {
        case 200:
          // Document successfully removed
          return({
            status: 'done',
            msg: 'Document successfully removed.',
            response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
          });
          break;
        case 202:
          // Request was accepted, but changes are not yet stored on disk
          return({
            status: 'pending',
            msg: 'Request received, but not applied.',
            response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
          });
          break;
        case 401:
          // Write privileges required
          return({
            status: 'error',
            msg: 'Write privileges required!'
          });
          break;
        case 404:
          // Specified database or document ID doesn’t exists
          return({
            status: 'error',
            msg: 'Specified database or document ID doesn’t exists!'
          });
          break;
        case 409:
          // Specified revision is not the latest for target document
          return({
            status: 'error',
            msg: 'Specified revision is not the latest for target document!'
          });
          break;
        default:
          return({
            status: 'error',
            msg: 'Invalid response status: '+doc.status
          });
      }
    }
  }

  updateDoc(dbIdentifier,docIdentifier,newdata,callback) {
    var _this = this;

    if(!newdata || !newdata.rev || !dbIdentifier || !docIdentifier) {
      throw new Error('Invalid Arguments given!');
    }

    if(_isset(callback,'function')) {
      Ajax({
        async: true,
        url: _this._build()+dbIdentifier+'/'+docIdentifier,
        method: 'PUT',
        contentType: 'application/json',
        data: JSON.stringify(newdata),
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        },
        complete: function(doc,status) {
          switch (doc.status) {
            case 201:
              // Document created and stored on disk
              callback({
                status: 'done',
                msg: 'Document created and stored on disk.',
                response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
              });
              break;
            case 202:
              // Document data accepted, but not yet stored on disk
              callback({
                status: 'pending',
                msg: 'Data received, but not stored.',
                response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
              });
              break;
            case 401:
              // Write privileges required
              callback({
                status: 'error',
                msg: 'Write privileges required!'
              });
              break;
            case 404:
              // Specified database or document ID doesn’t exists
              callback({
                status: 'error',
                msg: 'Specified database or document ID doesn’t exists!'
              });
              break;
            case 409:
              // Document with the specified ID already exists or specified revision is not latest for target document
              callback({
                status: 'error',
                msg: 'Specified revision is not the latest for target document!'
              });
              break;
            default:
              callback({
                status: 'error',
                msg: 'Invalid response status: '+doc.status
              });
          }
        }
      });
    } else {
      var doc = Ajax({
        async: false,
        url: _this._build()+dbIdentifier+'/'+docIdentifier,
        method: 'PUT',
        contentType: 'application/json',
        data: JSON.stringify(newdata),
        beforeSend: function(xhr) {
          if(_this.auth.name !== '' && _this.auth.password !== '') {
            xhr.setRequestHeader ("Authorization", "Basic " + btoa(_this.auth.name+":"+_this.auth.password));
          }
        }
      });

      switch (doc.status) {
        case 201:
          // Document created and stored on disk
          return({
            status: 'done',
            msg: 'Document created and stored on disk.',
            response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
          });
          break;
        case 202:
          // Document data accepted, but not yet stored on disk
          return({
            status: 'pending',
            msg: 'Data received, but not stored.',
            response: (typeof doc.responseJSON == 'object'?doc.responseJSON:doc.responseText)
          });
          break;
        case 401:
          // Write privileges required
          return({
            status: 'error',
            msg: 'Write privileges required!'
          });
          break;
        case 404:
          // Specified database or document ID doesn’t exists
          return({
            status: 'error',
            msg: 'Specified database or document ID doesn’t exists!'
          });
          break;
        case 409:
          // Document with the specified ID already exists or specified revision is not latest for target document
          return({
            status: 'error',
            msg: 'Specified revision is not the latest for target document!'
          });
          break;
        default:
          return({
            status: 'error',
            msg: 'Invalid response status: '+doc.status
          });
      }
    }
  }
}
